@@ -416,13 +416,25 @@ pub trait Serialize {
         Ok(())
     }
 
+    /// Checks if the current value that would be serialized represents an
+    /// optional value.
+    ///
+    /// This can be used by an emitter to skip over values that are currently
+    /// in the optional state.  For instance `Option<T>` returns `true` here if
+    /// the value is `None` and the struct emitter created by the `derive` feature
+    /// will skip over these if `#[deser(skip_serializing_optionals)]` is set on
+    /// the struct.
+    fn is_optional(&self) -> bool {
+        false
+    }
+
     /// Hidden internal trait method to allow specializations of bytes.
     ///
     /// This method is used by `u8` and `Vec<T>` / `&[T]` to achieve special
     /// casing of bytes for the serialization system.  It allows a vector of
     /// bytes to be emitted as `Chunk::Bytes` rather than a `Seq`.
     #[doc(hidden)]
-    fn __private_slice_as_bytes(_val: &[Self]) -> Option<&[u8]>
+    fn __private_slice_as_bytes(_val: &[Self]) -> Option<Cow<'_, [u8]>>
     where
         Self: Sized,
     {
